<?php

// $Id:


/**
 * @file
 * This module adds supports for sending emails when broken links are detected by the linkchecker module.
 *
 * Developed by Rahul Singla, http://www.rahulsingla.com/.
 */

/**
 * Implementation of hook_mail().
 */
function linkchecker_mail($key, &$message, $params) {
  $language = $message['language'];

  switch ($key) {
    case 'broken':
      $variables = linkchecker_mail_tokens($params['account'], $language);
      $message['subject'] .= linkchecker_mail_text($key . '_subject', $language, $variables);
      $message['body'][] = linkchecker_mail_text($key . '_body', $language, $variables);
      break;
  }
}

function linkchecker_mail_text($key, $language = NULL, $variables = array()) {
  $langcode = isset($language) ? $language->language : NULL;

  if ($admin_setting = variable_get('linkchecker_mail_' . $key, FALSE)) {
    // An admin setting overrides the default string.
    return strtr($admin_setting, $variables);
  }
  else {
    // No override, return default string.
    switch ($key) {
      case 'broken_subject':
        return t('Content created by you has broken links at !site', $variables, $langcode);
      case 'broken_body':
        return t("!username,\n\nThank you for your participation at !site. Content created by you at !site has been found to contain broken links. Please consider visiting your User Page and checking for broken links at: !user_link.\n\n\n--  !site team", $variables, $langcode);
    }
  }
}

function linkchecker_mail_tokens($account, $language) {
  global $base_url;
  $tokens = array(
      '!username' => $account->name,
      '!user_link' => url('user/' . $account->uid . '/linkchecker'),
      '!site' => variable_get('site_name', 'Drupal'),
      '!uri' => $base_url,
      '!uri_brief' => preg_replace('!^https?://!', '', $base_url),
      '!mailto' => $account->mail,
      '!date' => format_date(time(), 'medium', '', NULL, $language->language));
  return $tokens;
}

function _linkchecker_cron_start_time() {
  static $time = NULL;
  if (!isset($time)) {
    $time = time();
  }
  return ($time);
}

function _linkchecker_notify_broken_links() {
  if (!variable_get('linkchecker_mail_broken', TRUE)) {
    return;
  }

  $time = _linkchecker_cron_start_time();
  $status = 1;

  $ignore_response_codes = preg_split('/(\r\n?|\n)/', variable_get('linkchecker_ignore_response_codes', "200\n302\n304\n401\n403"));
  $codes = db_escape_string(implode(',', $ignore_response_codes));

  // Search for broken links in nodes and comments and blocks of all users.
  $sql = "
  SELECT * FROM users u INNER JOIN (
    SELECT DISTINCT c.uid
      FROM linkchecker_links ll
        INNER JOIN linkchecker_comments lc
          ON ll.last_checked >= $time AND ll.status = $status AND ll.code NOT IN ($codes) AND ll.lid = lc.lid
        INNER JOIN comments c ON lc.cid = c.cid
    UNION
    SELECT DISTINCT n.uid
      FROM linkchecker_links ll
        INNER JOIN linkchecker_nodes ln
          ON ll.last_checked >= $time AND ll.status = $status AND ll.code NOT IN ($codes) AND ll.lid = ln.lid
      INNER JOIN node n ON ln.nid = n.nid) q1 ON u.uid = q1.uid";

  $results = db_query($sql);
  while ($user = db_fetch_object($results)) {
    drupal_mail('linkchecker', 'broken', $user->mail, user_preferred_language($user), array(
        'account' => $user));
  }
}
